#include "camara.h"

void enviaDados (unsigned char leds, unsigned char cooler, unsigned char alarme)
{
	unsigned char saida;
	
	saida = 0x00;
	alarme = alarme << 7;
	saida = saida | alarme;
	cooler = cooler << 6;
	saida = saida | cooler;
	saida = saida | leds;
	
		
	outb(saida, LPT1);
}

unsigned char updateInfreds ()
{
	unsigned char infreds;
	
	infreds = inb(0x379);
	
	infreds = infreds >> 4;
	
	infreds = infreds & 0x07;
	
	return infreds;
}

void imprimeMensagem(const char *mensagem)
{
	int tamanho = strlen(mensagem);
	
	write(1, mensagem, tamanho);
}

void controleCamara ()
{									/*[ALARME, COOLER, P2, P1, L4, L3, L2, L1]*/
	static unsigned char leds;		/*[P2 (pino 7 | D5), P1 (pino 6 | D4), L4 (pino 5 | D3), L3 (pino 4 | D2), L2 (pino 3 | D1), L1 (pino 2 | D0)]*/ 
	static unsigned char cooler;	/*(pino 8 | D6)*/
	static unsigned char infreds;	/*[IR3, IR2, IR1]*/
	static unsigned char alarme;	/*(pino 9 | D7)*/
	
	int indice;
	int i;
	time_t temp_ini;
	
	char string[100];
	
	static Estado estado = reset;
	
	float tempo;
	
	switch (estado)
    {
    	case reset:
    		
    		abertura();
    		
    		leds = 0x00;
			infreds = 0x00;
			cooler = 0x00;
			
			enviaDados(leds, cooler, alarme);
			
			estado = aguardando;
			
			break;
		
		case aguardando:
			
			aplicacao();
			
			imprimeMensagem("Aguardando...\n\n");
			
			leds = leds | 0x01;			//acende o led L1
			enviaDados(leds, cooler, alarme);
			
			do
			{
				infreds = updateInfreds();
			}
			while(infreds != 0x01);
			
			estado = login;
			
			break;
			
		case login:
			
			indice = menuLogin();
			
			if(indice == ERROR)
			{
				estado = login_invalido;
			}
			else
			{
				estado = passagem_permitida;
			}
			
			break;
		
		case login_invalido:
			
			imprimeMensagem("Login inválido. Acesso negado.\n");
			
			leds = leds | 0x08;			//acende o led L4
			alarme = 0x01;				//soar alarme
			enviaDados(leds, cooler, alarme);
			
			esperarTempo(2);
			
			leds = leds & 0xF7;			//apaga o led L4
			alarme = 0x00;
			enviaDados(leds, cooler, alarme);
			
			
			strcpy(string, "Tentativa de acesso mal sucedida.\n");
			escreverLog(string, strlen(string));
			estado = login;
			
			break;
		
		case passagem_permitida:
			
			leds = leds | 0x04;	//acende o led L3
			enviaDados (leds, cooler, alarme);
			
			imprimeMensagem("Passagem Permitida.\n\n");
			
			sprintf(string, "%s entrou.", usuarios[indice].login);
			escreverLog(string, strlen(string));
			
			menuUsuario(indice);
			
			leds = leds | 0x10;	//acende o led P1
			enviaDados (leds, cooler, alarme);
			
			imprimeMensagem("Porta 1 aberta.\n\n");
			
			estado = ninguem_passou;
			
			temp_ini = time(NULL);
			
			do
			{
				infreds = updateInfreds();
				
				if (infreds == 0x02)
				{
					estado = esterilizar;
					break;
				}
			}
			while((time(NULL) - temp_ini) < TEMPO_ESPERA);
			
			break;
			
		case ninguem_passou:
			
			leds = leds & 0xFB;	//apaga o led L3
			leds = leds & 0xFE;	//apaga o led L1
			leds = leds & 0xEF;
			enviaDados (leds, cooler, alarme);
			
			imprimeMensagem("Ninguem passou.\n");
			
			alarme = 0x01;
			enviaDados(leds, cooler, alarme);
			
			esperarTempo(2);
			
			alarme = 0x00;
			enviaDados(leds, cooler, alarme);
			
			strcpy(string, "Ninguem passou.\n");
			escreverLog(string, strlen(string));
			
			estado = aguardando;
			
			break;
			
		case esterilizar:
			
			leds = leds & 0xEF;		//apaga P1		
			leds = leds & 0xFB;		//apaga L3
			leds = leds & 0xFE;		//apaga L1
			leds = leds | 0x02;		//acende L2
			alarme = 0x01;
			enviaDados (leds, cooler, alarme);
			
			esperarTempo(1);
			
			alarme = 0x00;
			cooler = 0x01;			//liga motor
			enviaDados (leds, cooler, alarme);
			
			tempo = calcularTempo(usuarios[indice].peso, usuarios[indice].altura);
			
			imprimeMensagem("Esterilizando...\n\n");
			
			printf("Tempo Necessario: %fs\n\n", tempo);
			
			esperarTempo((int) tempo);
						
			estado = encerrar_esterilizacao;
			
			break;
			
		case encerrar_esterilizacao:
			
			cooler = 0x00;			//desliga motor
			enviaDados (leds, cooler, alarme);
			
			for (i = 0; i < 3; i++)
			{
				alarme = 0x01;
				enviaDados (leds, cooler, alarme);			
				esperarTempo(1);
				
				alarme = 0x00;
				enviaDados (leds, cooler, alarme);
				esperarTempo(1);
			}
			
			leds = leds | 0x20;		//acende P2
			enviaDados (leds, cooler, alarme);
			
			system("clear||cls");
			
			imprimeMensagem("Esterilator:\n\n\n");
			imprimeMensagem("Porta 2 aberta\n\n");
			imprimeMensagem("Prossiga.\n\n");
			
			do
			{
				infreds = updateInfreds();
			}
			while(infreds != 0x04);
			
			estado = concluir_transito;		
			
			break;
		
		case concluir_transito:
		
			leds = leds & 0xDF;
			enviaDados(leds, cooler, alarme);
			
			strcpy(string, "Transito Concluido.\n");
			escreverLog(string, strlen(string));
			
			leds = leds & 0xFD;
			enviaDados(leds, cooler, alarme);
			
			system("clear||cls");
			
			estado = aguardando;
		
			break;
		
    };
}

void habilitaAcessoPorta ()
{
	if(ioperm(LPT1, 3, 1) ) /*Libera acesso para os endereços: 0x378, 0x379, 0x37A */
    {
        printf(" Acesso Negado. \n");
       
        if (errno == EINVAL)
        {
        	printf(" Numero ou endereco invalido.\n ");
        }
        else if (errno == EIO)
        {
        	printf(" This call is not supported.\n ");
        }
        else if (errno == EPERM)
        {
        	printf(" The calling process has insufficient privilege to call ioperm(); the CAP_SYS_RAWIO capability is required.\n ");
        }
        
         exit(1); /*Sai do programa */
    }
}

void teste() 
{
	unsigned char leds;
	unsigned char cooler;
	unsigned char alarme;
	
	leds = 0x00;
	cooler = 0x00;
	alarme = 0x00;

	enviaDados(leds, cooler, alarme);
	
	leds = updateInfreds(0x379);
	
	printf("%0x\n", leds);
}
